/*
 * poses.cpp
 *
 *  Created on: 20/05/2017
 *      Author: rescue
 */

#include <crosbot/geometry/poses.hpp>

#include <istream>
#include <sstream>
#include <string>

namespace crosbot {

// TODO: Fix parsing to match toString()
Pose2D Pose2D::parse(std::string str) {
    size_t f = str.find_first_of(')');
    if (f == str.npos) {
        return Pose2D();
    }
    Point2D p = Point2D::parse(str.substr(0, f+1));
    std::string aStr = str.substr(f+1);
    double a;
    if (sscanf(aStr.c_str(), "(%lf)", &a) == 1) {
    } else if (sscanf(aStr.c_str(), " (%lf)", &a) == 1) {
    } else if (sscanf(aStr.c_str(), "( %lf )", &a) == 1) {
    } else if (sscanf(aStr.c_str(), " ( %lf )", &a) == 1) {
    } else {
        a = 0;
    }

    return Pose2D(p, DEG2RAD(a));
}

Pose3D Pose3D::parse(std::string str) {
    // Check for starting "pose" string
    if (str.find("Pose") != std::string::npos) {
        // Parse Point
        int start = str.find_first_of('(');
        int end = str.find_first_of(')');
        std::string pointStr = str.substr(start+1, end - start);
        Point3D point = Point3D::parse(pointStr);

        // Parse Quaternion
        std::string quatStr = str.substr(end+2, str.npos);
        std::istringstream iss(quatStr);
        double yaw, pitch, roll;
        iss >> yaw;
        iss.get();
        iss >> pitch;
        iss.get();
        iss >> roll;
        Quaternion quat;
        quat.setYPR(yaw, pitch, roll);

        return Pose3D(point, quat);
    } else {
        // Use legacy parsing
        size_t f = str.find_first_of(')');
        if (f == str.npos) {
            return Pose3D();
        }
        ++f;
        Point3D p = Point3D::parse(str.substr(0, f));
        while (f < str.size() && isspace(str[f])) {
            ++f;
        }
        Quaternion q = Quaternion::parse(str.substr(f));
        return Pose3D(p, q);
    }

    return Pose3D();
}

} // namespace crosbot

