/*
 * particle.hpp
 *
 *  Created on: 20/02/2012
 *      Author: rescue
 */

#ifndef CROSBOT_FASTSLAM_PARTICLE_HPP_
#define CROSBOT_FASTSLAM_PARTICLE_HPP_

#include <crosbot_fastslam/heightmap.hpp>
#include <crosbot_map/map.hpp>

namespace crosbot {

namespace fastslam {

class Particle;
typedef Handle<Particle> ParticlePtr;
class Particle : public HandledObject {
public:
	struct History {
	public:
		Pose pose;
		bool restart;
		MapCloudPtr cloud;

		History(Pose pose, MapCloudPtr cloud = NULL, bool restart = false) :
			pose(pose), restart(restart), cloud(cloud)
		{}
	};

protected:
	Pose pose;
	Pose trackerPose;
	Map::TagListPtr tags;
	FastSLAMParameters& parameters;
	std::vector<History> history;

	HeightMultiMapPtr map;

	ReadWriteLock rwLock;

	tf::Transform poseTransform;
	double weight;
	MapCloudPtr motionCloud;

public:
	Particle(FastSLAMParameters& parameters);
	Particle(ParticlePtr parent, bool shallow = false);
	~Particle();

	/**
	 * Add's a new tag using robot-relative position
	 */
	void addTag(TagPtr tag, bool checkForDuplicate = false);

	/**
	 * Load a tag directly - no robot relative position.
	 * THIS IS GENERALLY UNSAFE, but useful when initially loading a map.
	 * The tags will be placed at the same global pose in all particles
	 */
	void unsafeAddTag(TagPtr tag);

	void applyMotion(tf::Transform motion, MapCloudPtr cloud, bool calculateWeight = true, double gain = DEFAULT_GAINVALUE);

	void update(MapCloudPtr cloud);
	inline MapCloudPtr getLatestUpdate() {
	    return motionCloud;
	}
	PathPtr getPath();

	inline void setPose(Pose pose) {
		Lock lock(rwLock, true);
		history.push_back(History(pose, NULL, true));
		this->pose = pose;
		this->poseTransform = this->pose.toTF();
	}

	inline Pose getPose() {
		Lock lock(rwLock);
		return pose;
	}

	inline Pose getPose(Pose& tracker) {
		Lock lock(rwLock);
		tracker = trackerPose;
		return pose;
	}

	/**
	 * Returns the current map. Care must be taken to keep everything else of the particle in sync
	 */
	inline HeightMultiMapPtr getMap() {
	    return map;
	}

	friend class FastSLAMMap;
	friend class FastSLAMRender;
	friend class crosbot::serialization::Serializer<Particle>;
};

} // namespace crosbot

} // namespace fastslam

#endif /* CROSBOT_FASTSLAM_PARTICLE_HPP_ */
